package com.example.permissioncheck;

import android.Manifest;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;

public class MainActivity extends AppCompatActivity {
    /*
     * どのパーミッション要求か識別するためのコード
     */
    public static final int PERMISSION_REQUEST_CODE = 1;

    @Override
    protected void onNewIntent(Intent intent) {
        // ※画面の更新
        // カメラ処理（省略）
        Log.v("MainActivity", "onNewIntent実行");
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        findViewById(R.id.button).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                // パーミッションが許諾されていればPackageManager.PERMISSION_GRANTED（0）、
                // 許諾されていなければPackageManager.PERMISSION_DENIED（-1）が返却される
                if(ContextCompat.checkSelfPermission(MainActivity.this, Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED){
                    // 許諾ダイアログを表示する必要がある場合
                    if(ActivityCompat.shouldShowRequestPermissionRationale(MainActivity.this, Manifest.permission.CAMERA)){
                        // ここに独自処理
                        // ユーザがダイアログで許諾を拒否している場合は許可を求める理由を表示する
                        Log.v("MainActivity", "権限が必要である事をユーザに伝える");
                        return;
                    } else {
                        // パーミッション依頼
                        Log.v("MainActivity", "権限要求");
                        ActivityCompat.requestPermissions(
                                MainActivity.this,
                                new String[]{Manifest.permission.CAMERA},
                                PERMISSION_REQUEST_CODE
                        );
                        return;
                    }
                }
                // カメラ処理（省略）
                Log.v("MainActivity", "権限有り、カメラ処理実行");
            }
        });
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        switch(requestCode){
            // 権限要求が複数パターンある場合はこのリクエストコードで処理を切り分ける
            case PERMISSION_REQUEST_CODE:
                // 権限が付与された場合
                if(grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED){
                    Log.v("MainActivity", "権限取得成功");
                    // 画面を更新する等の処理
                    Intent intent = new Intent(getApplication(), MainActivity.class);
                    intent.setFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP);
                    startActivity(intent);
                } else {
                    // ※独自処理
                    // 権限が付与されなかった場合
                    // 権限が必要な理由を表示する
                    Log.v("MainActivity", "権限が拒否された");
                }
                return;
        }
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
    }
}
