package com.example.splashstart;

import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.PersistableBundle;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.view.animation.AlphaAnimation;
import android.view.animation.Animation;

public class SplashActivity extends AppCompatActivity {
    private static final String TAG = "SplashActivity";
    private View[] labels = new View[3];
    private AlphaAnimation animation = null;
    private int animationNo = 0;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        animation = new AlphaAnimation(0.1f, 1);	// 透過の設定。
        animation.setDuration(3000);	// 1アニメーションにかける時間。
        animation.setAnimationListener(new Animation.AnimationListener(){
            // アニメーション終了終了時の処理。必要に応じて次のアニメーションを開始する。
            @Override
            public void onAnimationEnd(Animation animation) {
                labels[animationNo].setAnimation(null);	// アニメーションを停止。
                ++animationNo;	// 次のViewに対してアニメーション設定する。
                if(animationNo < labels.length){
                    labels[animationNo].setAlpha(1);	// 見える状態にして…
                    labels[animationNo].startAnimation(animation);	// …アニメーション開始。
                }
            }
            @Override
            public void onAnimationRepeat(Animation animation) {
            }
            @Override
            public void onAnimationStart(Animation animation) {
            }
        });
        setContentView(R.layout.activity_splash);
        // アニメーション
        labels[0] = findViewById(R.id.textView2);
        labels[1] = findViewById(R.id.textView3);
        labels[2] = findViewById(R.id.textView4);
        for(int i = 1; i < labels.length; ++i){
            labels[i].setAlpha(0);	// 一旦完全に透明にして、見えないようにする。
        }
        labels[0].startAnimation(animation);
        // 画面上のアニメーションと並行して、時間の掛かる初期化処理を別スレッドで行う。
        new Thread(new Runnable(){
            public void run(){
                // 時間の掛かる処理。
                for(int i = 0; i < 10; ++i){
                    Log.v(TAG, "processing:" + i);
                    try{
                        Thread.sleep(1000);
                    }catch(Exception e){
                    }
                }
                // 初期化処理が終わった事を、SharedPreferencesに記録する。
                SharedPreferences prefs = getSharedPreferences("settings", Activity.MODE_PRIVATE);
                SharedPreferences.Editor editor = prefs.edit();
                editor.putBoolean("initFlg", true).commit();
                // 全Viewのアニメーションを止める。
                for(int i = 0; i < labels.length; ++i){
                    labels[i].setAnimation(null);
                }
                // メイン画面に遷移する。
                startActivity(new Intent(SplashActivity.this, MainActivity.class));
                // スプラッシュ画面のほうは終了する。
                SplashActivity.this.finish();
            }
        }).start();
    }
}
