package com.example.datasavetiming;

import android.Manifest;
import android.content.Context;
import android.content.pm.PackageManager;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.CompoundButton;
import android.widget.Spinner;
import android.widget.Toast;
import android.widget.ToggleButton;

import java.util.ArrayList;
import java.util.List;

public class MainActivity extends AppCompatActivity implements LocationListener {
    private static final int ON_CHANGE = 0;
    private static final int ON_REC_NO = 1;
    private int saveMode;
    private LocationManager mLocationManager;
    private List<Location> locationList = new ArrayList<Location>();
    ToggleButton toggle;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        // 位置情報取得の準備。
        mLocationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
        // トグルボタンを用意し、位置情報取得をon・offできるようにする
        toggle = (ToggleButton) findViewById(R.id.toggleButton);
        toggle.setChecked(false);
        toggle.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (isChecked) {
                    // Android 6.0 API Level 23以降用権限チェック処理
                    if (ActivityCompat.checkSelfPermission(MainActivity.this, Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                        if (!ActivityCompat.shouldShowRequestPermissionRationale(MainActivity.this, Manifest.permission.ACCESS_COARSE_LOCATION)) {
                            ActivityCompat.requestPermissions(MainActivity.this, new String[]{Manifest.permission.ACCESS_COARSE_LOCATION}, 9000);
                        }
                        return;
                    }
                    mLocationManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 0, 0, MainActivity.this);
                } else {
                    if (mLocationManager != null) {
                        mLocationManager.removeUpdates(MainActivity.this);
                    }
                }
            }
        });

        // スピナーを用意し、位置情報の記録タイミングを選択できるようにする
        ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        adapter.add("位置情報更新時");
        adapter.add("10件取得時");
        Spinner spinner = (Spinner) findViewById(R.id.spinner);
        spinner.setAdapter(adapter);
        spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                switch (position) {
                    case ON_CHANGE:
                        // 位置情報が更新されるたびに保存する
                        saveMode = ON_CHANGE;
                        break;
                    case ON_REC_NO:
                        // 位置情報が10件取得できたら保存する
                        saveMode = ON_REC_NO;
                        break;
                    default:
                        break;
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {
            }
        });
    }

    @Override
    protected void onPause() {
        super.onPause();
        // アプリが非アクティブになったら位置情報の取得を止める
        if (mLocationManager != null) {
            // Android 6.0 API Level 23以降用権限チェック処理
            if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                return;
            }
            mLocationManager.removeUpdates(this);
            toggle.setChecked(false);
        }
    }

    @Override
    public void onLocationChanged(Location location) {
        locationList.add(location);
        if(saveMode == ON_CHANGE){
            appendFile();
        } else if (saveMode == ON_REC_NO) {
            if (locationList.size() >= 10) {
               appendFile();
            }
        }
    }

    @Override
    public void onProviderDisabled(String provider) {
    }
    @Override
    public void onProviderEnabled(String provider) {
    }
    @Override
    public void onStatusChanged(String provider, int status, Bundle extras) {
    }

    public void appendFile(){
        // ArrayListに保存している情報の数だけ繰り返し、ファイルに追記していく
        for(Location location : locationList) {
            // 位置情報をファイルに保存する
        }
        // 保存後、ArrayListをクリアする
        locationList.clear();
    }

    // Android 6.0 API Level 23以降用権限チェック処理
    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        if (requestCode == 9000 && grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
            ((ToggleButton) findViewById(R.id.toggleButton)).setChecked(false);
        }
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
    }
}
