package com.example.saveandshowerror;

import android.content.Context;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;

public class MainActivity extends AppCompatActivity {
    private Thread.UncaughtExceptionHandler defaultHandler =  null;
    // エラーを保存するファイル
    private static final String ERR_FILE_NAME = "errFile.txt";
    FileOutputStream out = null;
    FileInputStream in = null;
    BufferedReader reader = null;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        // ボタンの処理：エラーを発生させる
        findViewById(R.id.button).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                throw new RuntimeException();
            }
        });
        // デフォルトの例外ハンドラへの参照を保存する
        defaultHandler = Thread.getDefaultUncaughtExceptionHandler();
        // 自前の例外処理を定義する
        Thread.setDefaultUncaughtExceptionHandler(new Thread.UncaughtExceptionHandler(){
            @Override
            public void uncaughtException(Thread thread, Throwable ex) {
                // ファイルに例外クラス名を保存しておく
                try {
                    out = openFileOutput(ERR_FILE_NAME, Context.MODE_PRIVATE);
                    out.write(ex.getClass().getName().getBytes());
                    out.close();
                } catch (Exception e) {
                    if(out != null) try {out.close();} catch (Exception ignore){}
                }
                // エラー内容の保存が終わったら
                // 本来の（デフォルトの）例外処理を呼ぶ
                defaultHandler.uncaughtException(thread, ex);
            }
        });

        // アプリのonCreate時に、前回のエラー記録があるかチェックする
        File errFile = getFileStreamPath(ERR_FILE_NAME);
        // エラーを記録したファイルがあったら
        // ダイアログで表示し、ファイルを削除する
        if (errFile.exists()) {
            StringBuilder strBuff = new StringBuilder("前回起動時に以下エラーが発生しました。\n");
            try {
                in = new FileInputStream(errFile);
                reader = new BufferedReader(new InputStreamReader(in));
                String line = null;
                while((line = reader.readLine()) != null){
                    strBuff.append(line).append("\n");
                }
                reader.close();
                in.close();
                new AlertDialog.Builder(MainActivity.this)
                        .setMessage(strBuff.toString())
                        .setPositiveButton("ok", null)
                        .show();
                // ファイルを削除。次回からはダイアログを表示しない
                deleteFile(ERR_FILE_NAME);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}
