package com.example.userxjava;

import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.util.Log;

import rx.Observable;
import rx.Subscriber;
import rx.android.schedulers.AndroidSchedulers;
import rx.schedulers.Schedulers;
import rx.subscriptions.CompositeSubscription;

public class MyFragment extends Fragment {
    private static final String TAG = MyFragment.class.getSimpleName();

    private final CompositeSubscription mCompositeSubscription = new CompositeSubscription();

    public MyFragment() {
    }

    public static MyFragment newInstance() {
        MyFragment fragment = new MyFragment();
        Bundle args = new Bundle();
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onStart() {
        super.onStart();
        // Observableを追加していく。追加しただけ並行処理される
        mCompositeSubscription.add(Observable.create(new Observable.OnSubscribe<String>(){

                @Override
                public void call(Subscriber<? super String> subscriber) {
                    Log.v(TAG, "call");
                    // 例えば通信を実行して、複数データを受け取るような場合、受け取るたびにonNextで処理が可能
                    for(int i = 0; i < 3; ++i){
                        subscriber.onNext("success:" + i);
                        try {
                            Thread.sleep(1000);
                        } catch (InterruptedException e) {
                        }
                    }
                    // 全ての処理が完了したらonCompletedを呼ぶ
                    subscriber.onCompleted();
                }

            })
            .subscribeOn(Schedulers.newThread())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe(new Subscriber<String>(){
                @Override
                public void onCompleted() {
                    Log.v(TAG, "onCompleted");
                }

                @Override
                public void onError(Throwable e) {
                    Log.v(TAG, "onCompleted");
                }

                @Override
                public void onNext(String s) {
                    // FragmentやActivityが破棄された場合は、
                    // このメソッドは呼ばれない
                    Log.v(TAG, "onNext(" + s  + ")");
                }
            })
        );
    }

    @Override
    public void onDestroy() {
        mCompositeSubscription.unsubscribe();
        super.onDestroy();
    }
}
