package com.example.randtimingprocess;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.SystemClock;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.widget.Toast;

import java.util.Random;

public class MainActivity extends AppCompatActivity {
    public static final String ACTION_REPEAT_CONNECTION = "com.example.action.ACTION_REPEAT_CONNECTION";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // 通信開始用Intentを補足するためのフィルタを用意
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(ACTION_REPEAT_CONNECTION);

        // ブロードキャストレシーバを用意して、通信開始のIntentを受け取る
        MainActivity.this.registerReceiver(new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                if(ACTION_REPEAT_CONNECTION.equals(intent.getAction())){
                    Toast.makeText(context, "ここで通信処理を行う", Toast.LENGTH_SHORT).show();
                    // 乱数を用意し、次の実行までの時間にゆらぎを与える
                    Random rand = new Random(System.currentTimeMillis());
                    // 「10分 + 1～3分」後に次の通信を行うようスケジューリング
                    startAlarm(
                            // 10分
                            (1000 * 60 * 10)
                            // 1～3分（ランダム）
                            + (1000 * 60 * (rand.nextInt(3) + 1))
                    );
                }
            }
        }, intentFilter);
        // 初回の通信はすぐ（0ミリ秒後）に実行する
        startAlarm(0);
    }

    // ｘミリ秒後に次回の通信開始をスケジューリングする
    public void startAlarm(int millisecOffset){
        AlarmManager alarmManager = (AlarmManager)getSystemService(Context.ALARM_SERVICE);
        Intent intent = new Intent(ACTION_REPEAT_CONNECTION);
        PendingIntent pendingIntent = PendingIntent.getBroadcast(this, 0, intent, 0);

        alarmManager.set(AlarmManager.ELAPSED_REALTIME, SystemClock.elapsedRealtime() + millisecOffset, pendingIntent);

        // Android 4.4 API Level19かそれ以上の場合に、正確なアラームしていしたい場合は以下コードを使用する
//        alarmManager.setExact(AlarmManager.ELAPSED_REALTIME, SystemClock.elapsedRealtime() + millisecOffset, pendingIntent);
    }
}
