package com.example.sendmultidata;

import android.Manifest;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.IBinder;
import android.support.v4.app.ActivityCompat;
import android.widget.Toast;

public class GpsLoggerService extends Service implements LocationListener {
    private LocationManager mLocationManager = null;

    @Override
    public void onCreate() {
        super.onCreate();
        mLocationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
    }

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        Toast.makeText(this, "位置情報取得開始", Toast.LENGTH_SHORT).show();
        // Android 6.0 API Level 23以降用権限チェック処理
        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            return super.onStartCommand(intent, flags, startId);
        }
        mLocationManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 0, 0, this);
        return super.onStartCommand(intent, flags, startId);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        Toast.makeText(this, "位置情報取得終了", Toast.LENGTH_SHORT).show();
        if (mLocationManager != null) {
            // Android 6.0 API Level 23以降用権限チェック処理
            if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                return;
            }
            mLocationManager.removeUpdates(this);
        }
    }
    @Override
    public void onLocationChanged(Location location) {
        Toast.makeText(this, "位置情報取得, lat:" + location.getLatitude() + ", lon:" + location.getLongitude(), Toast.LENGTH_SHORT).show();
        StringBuilder gpsRecord = new StringBuilder();
        gpsRecord
                .append(Utils.formatDate(System.currentTimeMillis()))
                .append(",").append(location.getLatitude())
                .append(",").append(location.getLongitude())
                .append("\n");
        // 【データファイルその２】取得日時、緯度、経度をカンマ区切りでファイルに追記する。
        Utils.appendToFile(this, "location.log", gpsRecord.toString());
    }
    @Override
    public void onProviderDisabled(String provider) {
    }
    @Override
    public void onProviderEnabled(String provider) {
    }
    @Override
    public void onStatusChanged(String provider, int status, Bundle extras) {
    }
}