package com.example.sendmultidata;

import android.content.Intent;
import android.os.AsyncTask;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

public class MainActivity extends AppCompatActivity {
    private boolean isGpsRunning = false;
    private Button gpsButton = null;
    private Intent gpsIntent = null;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // 通算ミリ秒から現在日時の文字列を生成する
        String nowDate = Utils.formatDate(System.currentTimeMillis());
        // 【データファイルその１】生成した文字列を、起動日時としてファイルに記録（追記）する
        // （注：onCreate時なので、厳密には「起動時」ではありません、縦横切り替えでも呼ばれます）
        Utils.appendToFile(this, "app_exec.log", nowDate + "\n");

        // 位置情報記録Service呼び出し用Intentを用意する
        gpsIntent = new Intent(this, GpsLoggerService.class);
        // 位置情報記録Serviceが起動しているかチェックする
        isGpsRunning = Utils.isServiceRunning(this, GpsLoggerService.class.getName());

        // ボタンの設定、位置情報記録サービスをon・offする
        gpsButton = (Button)findViewById(R.id.button1);
        gpsButton.setOnClickListener(new View.OnClickListener(){
            public void onClick(View view) {
                if(isGpsRunning){
                    stopService(gpsIntent);
                }else{
                    startService(gpsIntent);
                }
                isGpsRunning = !isGpsRunning;
                gpsButton.setText((isGpsRunning) ? "位置情報取得停止" : "位置情報取得開始");
            }
        });
        // 位置情報記録サービスの状態に応じたボタンラベルを設定
        gpsButton.setText((isGpsRunning) ? "位置情報取得停止" : "位置情報取得開始");

        // アプリ起動ログが10件に達していたら、位置情報と一緒にサーバ送信する
        // アプリ起動日時は1レコード24バイトで記録しているので、
        // ファイルサイズを24で割ってレコード数を取得
        if((Utils.getFileSize(this, "app_exec.log") / 24) > 9){
            // AsyncTaskを用いて、別スレッドでサーバ通信を実施
            AsyncFileSender fileSender = new AsyncFileSender();
            fileSender.execute();
        }
    }

    // 非同期でサーバ通信を行うためのAsyncTask
    private class AsyncFileSender extends AsyncTask<Void, Void, Boolean> {
        final String boundary = "qazwsxedcrfvtgbyhn";
        final String[] fileList = {"app_exec.log", "location.log"};
        @Override
        protected Boolean doInBackground(Void... params) {
            // 通信やファイル処理に必要な各種要素
            HttpURLConnection conn = null;
            InputStream in = null;
            OutputStream out = null;
            URL urlObj;
            InputStream fileIn = null;
            try {
                int size = 0;
                // ファイル送信先のURLをここで指定
                urlObj = new URL("http://myserver.co.jp/uploadFiles");
                conn = (HttpURLConnection)urlObj.openConnection();
                conn.addRequestProperty("Content-Type", "multipart/form-data; boundary=" + boundary);
                conn.setRequestMethod("POST");
                conn.setDoOutput(true);
                out = conn.getOutputStream();

                // サーバに送るファイルの数だけ繰り返し
                for(int i = 0; i < fileList.length; ++i){
                    byte[] byteBuffer = new byte[1024];
                    StringBuilder postData = new StringBuilder();

                    // 複数データの区切りであるバウンダリ情報、ファイル名、データ種別などを送る
                    postData
                            .append("--").append(boundary).append("\r\n")
                            .append("Content-Disposition: form-data; name=\"data").append(i)
                            .append("\"; filename=\"").append(fileList[i]).append("\"\r\n")
                            .append("Content-Type: application/octet-stream\r\n\r\n");
                    out.write(postData.toString().getBytes());

                    // ファイルを開いて、その内容を取得しつつ、通信でサーバに送る
                    // ※注※ このサンプルでは排他制御を入れていません
                    // 実際のアプリではファイルアクセスを制御しましょう
                    fileIn = openFileInput(fileList[i]);
                    while((size = fileIn.read(byteBuffer)) > 0){
                        out.write(byteBuffer, 0, size);
                    }
                    fileIn.close();
                    out.write("\r\n".getBytes());
                    deleteFile(fileList[i]);
                }

                // すべてのファイルの情報を送ったら、末尾のバウンダリ情報を送る
                out.write(("--" + boundary + "--\r\n").getBytes());
                out.close();

                // サーバのレスポンスコードを判定する場合はこの値を使用
                int respCode = conn.getResponseCode();
                in = conn.getInputStream();
                byte[] byteBuffer = new byte[1024];

                // サーバのレスポンスボディを受け取る場合はここで処理
                while((size = in.read(byteBuffer)) != -1){
                }
                return true;
            } catch (MalformedURLException e) {
                // URL形式に問題があった場合のエラー
            } catch (IOException e) {
                // 入出力関連エラー
            }
            return false;
        }
    }
}
