package com.example.usebackoff;

import android.app.AlarmManager;
import android.app.IntentService;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.SystemClock;
import android.util.Log;

public class BackgroundService extends IntentService{
    private static final String TAG = "BackgroundService";
    private static final String PREF_NAME = "com.example.backoffsample";
    private static final String PREF_KEY_BACKOFF = "pref_key_backoff";
    public static final long INITIAL_BACKOFF_MS = 30 * 1000;	// リトライ間隔初期値は30秒
    public BackgroundService() {
        super("BackgroundService");
    }
    public static void setBackoff(Context context, long timeMs){
        final SharedPreferences prefs = context.getSharedPreferences(PREF_NAME, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();
        editor.putLong(PREF_KEY_BACKOFF, timeMs);
        editor.commit();
    }
    public static long getBackoff(Context context){
        final SharedPreferences prefs = context.getSharedPreferences(PREF_NAME, Context.MODE_PRIVATE);
        return prefs.getLong(PREF_KEY_BACKOFF, 0);
    }
    @Override
    protected void onHandleIntent(Intent intent) {
        Log.v(TAG, "バックグラウンド処理開始");
        // 時間の掛かる処理。実際はここで通信などを行う
        try{
            Thread.sleep(3 * 1000);
        }catch(Exception e){
        }
        Log.v(TAG, "バックグラウンド処理終了");
        // 次のリトライまでの間隔をSharedPreferenceから取得
        long nextInterval = getBackoff(this);
        // リトライ間隔時間が0ミリ秒ならリトライをスケジューリングしない。実際は通信処理の結果などでリトライ要否を判定する
        if(nextInterval > 0){
            Log.v(TAG, "次のリトライまでの時間（ミリ秒）:" + nextInterval);
            // リトライ間隔時間経過したらこのIntentService自身を呼び出す。
            PendingIntent pendingIntent = PendingIntent.getService(this, 0, new Intent(this, BackgroundService.class), PendingIntent.FLAG_CANCEL_CURRENT);
            AlarmManager am = (AlarmManager)getSystemService(Context.ALARM_SERVICE);
            // Android 4.4 API Level 19からはset()でなくsetExact()メソッドを使用しないと正確な時間でなくなる
            am.setExact(AlarmManager.ELAPSED_REALTIME, SystemClock.elapsedRealtime() + nextInterval, pendingIntent);
            setBackoff(this, nextInterval * 2);	// リトライ間隔を倍にして再セット
        }else{
            Log.v(TAG, "次のリトライはセットせず");
        }
    }
}
