package com.example.userangerequest;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

public class Utils {
    public static boolean resumeDownload(String url, String filePath){
        boolean isSuccess = false;
        byte[] byteBuffer = new byte[1024];
        HttpURLConnection conn = null;
        InputStream in = null;
        FileOutputStream fout = null;
        try {
            URL urlObj = new URL(url);
            conn = (HttpURLConnection)urlObj.openConnection();
            File tmpFile = new File(filePath + ".tmp");	// 「.tmp」付きのファイルがあれば中断されている
            if(tmpFile.exists()){
                // length()はファイルが存在しなければ0を返す
                conn.setRequestProperty("Range", "bytes=" + tmpFile.length() + "-");
            }else{
                tmpFile.createNewFile();
            }
            conn.setReadTimeout(60 * 1000);
            conn.setRequestMethod("GET");
            conn.connect();
            int respCode = conn.getResponseCode();
            boolean isRangeOk = ("bytes".equals(conn.getHeaderField("Accept-Ranges")));
            if(respCode == 200 || respCode == 206) {
                in = conn.getInputStream();
                fout = new FileOutputStream(tmpFile, isRangeOk);    // Rangeリクエストを受け付ければAppend
                int byteRead = 0;
                while ((byteRead = in.read(byteBuffer)) >= 0) {
                    fout.write(byteBuffer, 0, byteRead);
                }
                fout.flush();
                File saveFile = new File(filePath);
                if (saveFile.exists()) {    // ダウンロード済みのファイルがあれば削除する
                    saveFile.delete();
                }
                tmpFile.renameTo(saveFile);    // ファイル名から「.tmp」を外す。
                isSuccess = true;
            }
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if(fout != null) try {fout.close();} catch (IOException e) {}
            if(in != null) try {in.close();} catch (IOException e) {}
            if(conn != null) conn.disconnect();
        }
        return isSuccess;
    }
}
