package com.example.useencryption;

import android.content.ContentValues;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class MainActivity extends AppCompatActivity {
    private EditText passInput = null;
    private EditText dataInput = null;
    private Button encButton = null;
    private ListView list = null;
    private SQLiteDatabase db = null;
    private String password = null;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        // データベースの準備
        DatabaseHelper helper = new DatabaseHelper(this);
        db = helper.getWritableDatabase();

        // UI部品取得
        passInput = (EditText)findViewById(R.id.editText1);
        dataInput = (EditText)findViewById(R.id.editText2);
        encButton = (Button)findViewById(R.id.button1);
        list = (ListView)findViewById(R.id.listView1);

        // ボタンをタップした時の動作を定義。
        encButton.setOnClickListener(new View.OnClickListener(){
            public void onClick(View view) {
                // パスワードを取得、カラなら処理せずToastを表示する。
                password = passInput.getText().toString();
                if(password.isEmpty()){
                    Toast.makeText(MainActivity.this, "パスワードを入力してください", Toast.LENGTH_LONG).show();
                }else{
                    // 入力フィールドのテキストを取得、暗号化してデータベースに格納する
                    // 実際は暗号化から格納までを非同期に行うこと
                    insertEncData(dataInput.getText().toString());
                    Toast.makeText(MainActivity.this, "保存完了", Toast.LENGTH_SHORT).show();
                }
            }
        });

        // リスト内の要素をタップしたら暗号化されたデータを複合化し、Toastで表示する
        list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            public void onItemClick(AdapterView<?> parent, View view, int viewPos, long rowId) {
                // パスワードを取得、カラなら処理せずToastを表示する
                password = passInput.getText().toString();
                if(password.isEmpty()){
                    Toast.makeText(MainActivity.this, "パスワードを入力してください", Toast.LENGTH_LONG).show();
                    return;
                }
                Map<String, String> dataMap = (HashMap<String, String>)parent.getItemAtPosition(viewPos);
                final int targetId = Integer.parseInt(dataMap.get("firstLine"));
                final String decodedData = selectDecData(targetId);
                Toast.makeText(MainActivity.this, "デコードした値：" + decodedData, Toast.LENGTH_LONG).show();
                return;
            }
        });

        // データベース内のレコードをListViewに表示
        updateList();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if(db != null) db.close();
    }

    // バイト配列を16進数表記の文字列に変換する。
    public static String dumpBinary(byte[] rawData){
        if(rawData == null) return "null";
        StringBuilder buffer = new StringBuilder();
        for(int i = 0; i < rawData.length; ++i){
            buffer.append(String.format("%02x", rawData[i]));
        }
        return buffer.toString();
    }

    // 受け取った文字列を暗号化し、データベースに保存する
    private void insertEncData(String str){
        ContentValues values = new ContentValues();
        // 暗号化して値をContentValuesに格納
        try {
            values.put("crypted_data", CryptUtil.encryptData(password.toCharArray(), str.getBytes()));
        } catch (Exception e) {	// エラー処理は省略
        }
        // データベースに挿入
        db.insert("secret_data", null, values);
        // 画面上の一覧を更新
        updateList();
    }

    // データベースから_idの合致するレコードを取得し、複合化した文字列を返す
    private String selectDecData(int targetId){
        Cursor cursor = db.query("secret_data",
                new String[]{"crypted_data"},	// 取得する列
                "_id = ?", // where
                new String[]{Integer.toString(targetId)},	// parameter
                null, 	// group by
                null, 	// having
                null, 	// order by
                null	// limit
        );
        // プライマリキーを条件にselectするのでカーソルのループは不要
        cursor.moveToFirst();
        byte[] encryptedBinary = cursor.getBlob(0);
        try {
            byte[] decryptedBinary = CryptUtil.decryptData(password.toCharArray(), encryptedBinary);
            return new String(decryptedBinary);
        } catch (Exception e) {	// エラー処理は省略
        }
        return null;
    }

    private void updateList(){
        // データベースの値を格納するリストを用意
        List<Map<String, String>> dataList = new ArrayList<Map<String, String>>();
        // 画面上の一覧を更新するためにデータベースから値を取得する
        Cursor cursor = db.query("secret_data",
                new String[]{"_id", "crypted_data"},	// 取得する列
                null,	// where
                null, 	// parameter
                null, 	// group by
                null,	// having
                "_id asc", 	// order by
                null	// limit
        );
        if(cursor.moveToFirst()){
            // カーソルを先頭に移動し、レコードの数だけループ
            do{
                int recordId = cursor.getInt(0);
                String cryptedData = dumpBinary(cursor.getBlob(1));
                // ListViewのAdapterにあてはめるためにMapに格納する
                Map<String, String> data = new HashMap<String, String>();
                // 1行目はレコードID
                data.put("firstLine", Integer.toString(recordId));
                // 2行目は暗号化したデータ
                data.put("secondLine", cryptedData);
                dataList.add(data);
            }while(cursor.moveToNext());
        }
        cursor.close();	// カーソルは必ず閉じる
        SimpleAdapter adapter = new SimpleAdapter(this,
                dataList,
                android.R.layout.simple_list_item_2,
                new String[]{"firstLine", "secondLine"},
                new int[]{android.R.id.text1, android.R.id.text2}
        );
        list.setAdapter(adapter);
    }
}
