package com.example.sqlciphersample;

import android.content.ContentValues;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import android.widget.Toast;

import net.sqlcipher.Cursor;
import net.sqlcipher.database.SQLiteDatabase;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class MainActivity extends AppCompatActivity {
    private EditText input = null;
    private Button button = null;
    private ListView list = null;
    private SQLiteDatabase db = null;
    private String password = "opensesame";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // データベースの準備
        SQLiteDatabase.loadLibs(this);
        CipherDbHelper helper = new CipherDbHelper(this);
        db = helper.getWritableDatabase(password);

        // UI部品取得
        input = (EditText)findViewById(R.id.editText1);
        button = (Button)findViewById(R.id.button1);
        list = (ListView)findViewById(R.id.listView1);

        // ボタンをタップした時の動作を定義
        button.setOnClickListener(new View.OnClickListener(){
            public void onClick(View view) {
                // 入力フィールドのテキストを取得、暗号化してデータベースに格納する
                insertEncData(input.getText().toString());
                Toast.makeText(MainActivity.this, "保存完了", Toast.LENGTH_SHORT).show();
            }
        });

        // データベース内のレコードをListViewに表示
        updateList();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if(db != null) db.close();
    }

    // 受け取った文字列を暗号化し、データベースに保存する。※本来は非同期処理にするのが好ましい
    private void insertEncData(String str){
        ContentValues values = new ContentValues();
        // 値をContentValuesに格納
        values.put("crypted_data", str);
        // データベースに挿入
        db.insert("secret_data", null, values);
        // 画面上の一覧を更新
        updateList();
    }

    private void updateList(){
        // データベースの値を格納するリストを用意
        List<Map<String, String>> dataList = new ArrayList<Map<String, String>>();
        // 画面上の一覧を更新するためにデータベースから値を取得する
        Cursor cursor = db.query("secret_data",
                new String[]{"_id", "crypted_data"},	// 取得する列
                null, null, null, null, // where, parameter, group by, having
                "_id asc", 	// order by
                null	// limit
        );
        if(cursor.moveToFirst()){
            // カーソルを先頭に移動し、レコードの数だけループ
            do{
                int recordId = cursor.getInt(0);
                String cryptedData = cursor.getString(1);
                // ListViewのAdapterにあてはめるためにMapに格納する
                Map<String, String> data = new HashMap<String, String>();
                data.put("firstLine", Integer.toString(recordId));	// 1行目はレコードID
                data.put("secondLine", cryptedData);				// 2行目は暗号化したデータ
                dataList.add(data);
            }while(cursor.moveToNext());
        }
        cursor.close();	// カーソルは必ず閉じる
        SimpleAdapter adapter = new SimpleAdapter(this,
                dataList,
                android.R.layout.simple_list_item_2,
                new String[]{"firstLine", "secondLine"},
                new int[]{android.R.id.text1, android.R.id.text2}
        );
        list.setAdapter(adapter);
    }
}
