package com.example.presenterpattern;

import com.example.presenterpattern.model.AddressItem;
import com.example.presenterpattern.presenter.MainPresenter;
import com.example.presenterpattern.presenter.MainPresenterCallback;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertThat;
import static org.mockito.ArgumentCaptor.forClass;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

@RunWith(MockitoJUnitRunner.class)
public class MainPresenterTest {
    @Mock
    MainPresenterCallback mMainPresenterCallback;

    @InjectMocks
    private MainPresenter mMainPresenter;

    @Before
    public void setUp() throws Exception {
        mMainPresenter = new MainPresenter(mMainPresenterCallback);
    }

    @Test
    public void checkItem01() throws Exception {
        ArgumentCaptor<AddressItem> captor = forClass(AddressItem.class);
        mMainPresenter.convert("163-1490");
        Thread.sleep(100);
        verify(mMainPresenterCallback, times(1)).showAddress(captor.capture());
        assertThat(captor.getValue().name, is("東京都新宿区西新宿"));
    }

    @Test
    public void checkItem02() throws Exception {
        ArgumentCaptor<AddressItem> captor = forClass(AddressItem.class);
        mMainPresenter.convert("150-0042");
        Thread.sleep(100);
        verify(mMainPresenterCallback, times(1)).showAddress(captor.capture());
        assertThat(captor.getValue().name, is("東京都渋谷区宇田川町"));
    }

    @Test
    public void checkErrorMessage() throws Exception {
        ArgumentCaptor<String> captor = forClass(String.class);
        mMainPresenter.convert("169-0072");
        Thread.sleep(100);
        verify(mMainPresenterCallback, times(1)).showErrorMessage(captor.capture());
        assertThat(captor.getValue(), is("登録されていない住所です"));
    }
}
